package com.cesgroup.kingkong.web;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.cesgroup.common.utils.RSAEncryptUtil;
import com.cesgroup.common.utils.RestAPIResult;
import com.cesgroup.kingkong.entity.LogEntity;
import com.cesgroup.kingkong.service.LogService;
import com.cesgroup.kingkong.service.SecretKeyService;
import com.cesgroup.kingkong.service.impl.QueryService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;


@RestController
@RequestMapping("v1")
@Api("专利论文接口API说明")
public class SearchContrllor {

    @Autowired
    SecretKeyService secretKeyService;
    @Autowired
    LogService logService;

	@Autowired
	private QueryService queryService;
	
	@ApiOperation(value = "专利查询接口")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "sign", value = "将实际查询参数（id）经过加密之后的值", dataType = "String", paramType = "query") })
	@RequestMapping(value = "queryById", method = { RequestMethod.POST })
	public RestAPIResult<Object> query(String sign) throws Exception {
	    long before = new Date().getTime();
	    Map<String, Object> params = RSAEncryptUtil.resolveParamsForSign(sign, secretKeyService.getById(2));
	    String id = MapUtils.getString(params, "id");
		RestAPIResult<Object> panResult = new RestAPIResult<Object>(secretKeyService);
		Map<String, Object> result = null;
		if(StringUtils.isNotBlank(id)) {
			result =queryService.getById(id);
			String respCodeString = (String) result.get("resonCode");
			if (respCodeString.equals("200")) {
				panResult.isSuccess();
				panResult.setRespMsg("文档信息查询成功！");
				panResult.setRespData(result.get("result"));
				panResult.setRespCode(Integer.parseInt(respCodeString));
			} else if(respCodeString.equals("2011")) {
				panResult.isSuccess();
				panResult.setRespMsg("文档id不存在！");
				panResult.setRespData(result.get("result"));
				panResult.setRespCode(Integer.parseInt(respCodeString));
			}else if(respCodeString.equals("2012")) {
				panResult.isSuccess();
				panResult.setRespMsg("文档解析失败");
				panResult.setRespData(result.get("result"));
				panResult.setRespCode(Integer.parseInt(respCodeString));
			}else if(respCodeString.equals("1011")) {
				panResult.isSuccess();
				panResult.setRespMsg("文档查询超时！");
				panResult.setRespData(result.get("result"));
				panResult.setRespCode(Integer.parseInt(respCodeString));
			}else  {
				panResult.error();
				panResult.setRespMsg("其他错误！");
			}
		}else {
			panResult.error("查询参数错误");
			panResult.setRespCode(1012);
		}
        insertLog(before, "/v1/queryById");
		return panResult;
	}

    private void insertLog(long before, String name) {
        long after = new Date().getTime();
        long processing = after - before;
        LogEntity logEntity = new LogEntity();
        logEntity.setName(name);
        logEntity.setProcessTime(processing);
        logEntity.setCreateTime(new Date());
        logService.save(logEntity);
    }

    /**查询论文接口
	 * @return
	 * RestAPIResult<Object>
	 */
    @ApiOperation(value = "论文查询接口")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "sign", value = "将实际查询参数（id）经过加密之后的值", dataType = "String", paramType = "query") })
	@RequestMapping(value = "queryArticleInfo", method = { RequestMethod.POST })
	public RestAPIResult<Object> queryArticleInfo(String sign) throws Exception {
	    long before = new Date().getTime();
        Map<String, Object> params = RSAEncryptUtil.resolveParamsForSign(sign, secretKeyService.getById(2));

		RestAPIResult<Object> panResult = new RestAPIResult<Object>(secretKeyService);
		Map<String, Object> result = queryService.queryArticleInfo(params);
		if(result == null || result.size() <= 0){
		    // 未查出数据时，去掉机构名后重新查询
		    params.remove("affilorg");
		    result = queryService.queryArticleInfo(params);
        }

		Map<String, Object> resMap = new HashMap<>();
		resMap.put("result", parseResult(result));
		resMap.put("opensearch:totalResults", result.get("opensearch:totalResults"));
        panResult.setRespData(resMap);
        insertLog(before, "/v1/queryArticleInfo");
		return panResult;
	}

    /**
     * 整理返回信息
     * @return
     */
    private List<Map<String, Object>> parseResult(Map<String, Object> beforehand){
        if(beforehand != null && beforehand.size() > 0){
            List<Map<String, Object>> resultList = (List<Map<String, Object>>) beforehand.get("entry");
            List<Map<String, Object>> result = new ArrayList<>();
            for (Map<String, Object> map: resultList) {
                Map<String, Object> temp = new HashMap<>();
                temp.put("dc:creator", map.get("dc:creator"));
                temp.put("subtypeDescription", map.get("subtypeDescription"));
                temp.put("prism:publicationName", map.get("prism:publicationName"));
                temp.put("prism:volume", map.get("prism:volume"));
                temp.put("citedby-count", map.get("citedby-count"));
                temp.put("prism:pageRange", map.get("prism:pageRange"));
                temp.put("dc:title", map.get("dc:title"));
                temp.put("prism:coverDisplayDate", map.get("prism:coverDisplayDate"));
                temp.put("affiliation", map.get("affiliation"));
                result.add(temp);
            }

            return result;
        }
        return null;
    }
}
